# -*- coding: utf-8 -*-
"""
edx_rbac Django application initialization.
"""

from django.apps import AppConfig


class EdxRbacConfig(AppConfig):
    """
    Configuration for the edx_rbac Django application.
    """

    name = 'edx_rbac'
